package fungi

  1. Overview
  2. Docs

Module type Axiom.SpaceSource

Space

this is an ast mainly used when doing path finding or flow algorithms in the graph. It allows you to bring in your own implementation of the edge. Example valid structure are `Float` or `Int`.

type t
include Set.OrderedType with type t := t
val compare : t -> t -> int

A total ordering function over the set elements. This is a two-argument function f such that f e1 e2 is zero if the elements e1 and e2 are equal, f e1 e2 is strictly negative if e1 is smaller than e2, and f e1 e2 is strictly positive if e1 is greater than e2. Example: a suitable ordering function is the generic structural comparison function Stdlib.compare.

val zero : t
val one : t
val add : t -> t -> t
val sub : t -> t -> t
val min : t -> t -> t
val max : t -> t -> t
val neg : t -> t