Source file heap.ml
1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
424
425
426
427
428
429
430
431
432
433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
468
469
470
471
472
473
474
475
476
477
478
479
480
481
482
483
484
485
486
487
488
489
490
491
492
493
494
495
496
497
498
499
500
501
502
503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
523
524
525
526
527
528
529
530
531
532
533
534
535
536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
554
555
556
557
558
559
560
561
562
563
564
565
566
567
568
569
570
571
572
573
574
575
576
577
578
579
580
581
582
583
584
585
586
587
588
589
590
591
592
593
594
595
596
597
598
599
600
601
602
603
604
605
606
607
608
609
610
611
612
613
614
615
616
617
618
619
620
621
622
623
624
625
626
627
628
629
630
631
632
633
634
635
636
637
638
639
640
641
642
643
644
645
646
647
648
649
650
651
652
653
654
655
656
657
658
659
660
661
662
663
664
665
666
667
668
669
670
671
672
673
674
675
676
677
678
679
680
(**
{1:fibheap Fibonacci Heap Implementation }
Simplest Indexed Fibonacci (d-ary) heap
- Every node's key is less(default cmp) than or equal to its children
keys as given by the Entry.compare function
- The minimum|maximum element is always in the root list depending
on your compare function
- Unlike binary heaps, there's no enforced structure
- Duplicates are tolerated as they can be introduced by decrease and
increase operations
- A node can have any number of children
- Children can be added or removed freely
- no redundancy (if 2 nodes are the same their trees are assumed to
be the same)
- For simplicity we don't have a min pointer at the root list but it may
be added in the future, we so have to walk the root list to get the
min
*)
module type Ordinal = sig
type t
type order
val bind: t -> order
val compare: t -> t -> int
val order: order -> order -> int
end
module Surject(Inner: Set.OrderedType): Ordinal with type t = Inner.t and type order = Inner.t = struct
type t = Inner.t
type order = Inner.t
let bind e = e
let order = Inner.compare
let compare = Inner.compare
end
module type FibHeap = sig
type node
type order
type elts = { data: node; mutable churn: int; index: int; succ: elts list }
type t = elts list
val empty: t
val is_empty: t -> bool
val equal: node -> node -> bool
val minify: node -> node -> bool
val maxify: node -> node -> bool
val oequal: node -> node -> int
val degree: elts -> int
val cardinal: t -> int
val collapse: t -> node list
val churn_threshold: int ref
val instance: node -> int -> elts
val singleton: node -> t
val dedup: node -> elts list -> (elts list * node list)
val dedup_idx: node -> elts list -> elts list * (node * int) list
val dupcount: node -> t -> int
val to_seq: ?cmp:(node -> node -> bool) -> t -> node Seq.t
val of_seq: ?cmp:(node -> node -> bool) -> node Seq.t -> elts list
val of_list: ?cmp:(node -> node -> bool) -> node list -> t
val consolidate: ?cmp:(node -> node -> bool) -> t -> t
val mem: ?cmp:(node -> node -> bool) -> node -> t -> bool
val insert: ?cmp:(node -> node -> bool) -> node -> t -> t
val dupinsert: ?cmp:(node -> node -> bool) -> node -> t -> t
val peek: ?cmp:(node -> node -> bool) -> t -> elts
val peek_opt: ?cmp:(node -> node -> bool) -> t -> elts option
val merge: ?cmp:(node -> node -> bool) -> elts -> elts -> elts
val update: ?cmp:(node -> node -> bool) -> node -> elts -> elts list -> elts -> elts list -> elts list * elts list * bool
val increase: ?cmp:(node -> node -> bool) -> node -> node -> t -> t
val decrease: ?cmp:(node -> node -> bool) -> node -> node -> t -> t
val find: (node -> bool) -> elts list -> node
val find_opt: (node -> bool) -> elts list -> node option
end
(**
{2:create Create a Fibonacci Heap}
{@ocaml[
module F = Heap.MakeFibHeap (struct
(* the main type *)
type t = int
(* how to determine order *)
type order = int
let bind e = e
(* compare to orders *)
let order = Int.compare
(* compare to nodes *)
let compare = Int.compare
end);;
]}
*)
module MakeFibHeap(Entry: Ordinal): FibHeap with type node = Entry.t and type order = Entry.order = struct
type node = Entry.t
;;
type order = Entry.order
;;
type elts = { data: node; mutable churn: int; index: int; succ: elts list }
;;
type t = elts list
;;
let empty = []
;;
let is_empty = function
| [] -> true
| _ :: _ -> false
;;
let equal l r = (Entry.compare l r) = 0
;;
let oequal l r = (Entry.order (Entry.bind l) (Entry.bind r))
;;
let maxify l r = oequal l r = 1
;;
let minify l r = oequal l r = -1
;;
let churn_threshold = ref 8
;;
let instance pleaf idx = { data=pleaf; churn=0; index=idx; succ=[] }
;;
let mem ?(cmp=minify) pleaf ptree =
let rec fmem tree rem = match tree with
| [] -> (match rem with
| [] -> false
| branch :: left ->
(match branch with
| [] ->
(match left with
| [] -> false
| l :: r -> fmem l r)
| hd :: rest ->
(equal hd.data pleaf)
||
(if cmp hd.data pleaf then
fmem hd.succ (rest :: left)
else
fmem rest (hd.succ :: left))
)
)
| hd :: tail ->
(equal hd.data pleaf)
||
(if cmp hd.data pleaf then
fmem hd.succ (tail :: rem)
else
fmem tail (hd.succ :: rem))
in fmem ptree []
;;
let rec cardinal = function
| [] -> 0
| hd :: tail ->
(cardinal hd.succ) + 1 + (cardinal tail)
;;
let find f = function
| [] -> raise Not_found
| hd :: tail ->
if f hd.data then
hd.data
else
let rec mergefind tree rem = match tree with
| [] ->
(match rem with
| [] -> raise Not_found
| hd :: rest ->
if f hd.data then
hd.data
else
(mergefind rest hd.succ)
)
| hd :: tail ->
if f hd.data then
hd.data
else
(mergefind tail (hd.succ @ rem))
in mergefind hd.succ tail
;;
let find_opt f = function
| [] -> None
| hd :: tail ->
if f hd.data then
Some hd.data
else
let rec mergefind tree rem = match tree with
| [] ->
(match rem with
| [] -> None
| hd :: rest ->
if f hd.data then
Some hd.data
else
(mergefind rest hd.succ)
)
| hd :: tail ->
if f hd.data then
Some hd.data
else
(mergefind tail (hd.succ @ rem))
in mergefind hd.succ tail
;;
let dupcount pleaf tree =
let rec fdup stree acc = match stree with
| [] -> acc
| hd :: tl ->
if equal hd.data pleaf then
fdup tl @@ fdup hd.succ (acc + 1)
else
(fdup[@tailcall]) tl acc
in fdup tree 0
;;
let insert ?(cmp=minify) pleaf tree =
let rec pinsert pleaf dupc = function
| [] -> [ (instance pleaf dupc) ]
| (hd :: tail) ->
if equal hd.data pleaf then
let dupc' = dupc + 1 in
{ hd with succ= (pinsert pleaf (dupc') hd.succ) } :: tail
else if cmp hd.data pleaf then
{ hd with succ=(pinsert pleaf dupc hd.succ) } :: tail
else
hd :: (pinsert pleaf dupc tail)
in pinsert pleaf 0 tree
;;
let dupinsert ?(cmp=minify) pleaf tree =
let dc = (dupcount pleaf tree) + 1 in
let rec dupinsert pleaf dupc = function
| [] -> [ (instance pleaf dupc) ]
| (hd :: tail) ->
if equal hd.data pleaf then
if cmp hd.data pleaf then
{ hd with succ=(dupinsert pleaf (dupc) hd.succ) } :: tail
else
{ data=pleaf; churn=0; index=dupc;
succ=(dupinsert hd.data (hd.index) hd.succ)
} :: (tail)
else if cmp hd.data pleaf then
{ hd with succ=(dupinsert pleaf dupc hd.succ) } :: tail
else
hd :: (dupinsert pleaf dupc tail)
in dupinsert pleaf dc tree
;;
let singleton pleaf =
[ { data=pleaf; churn=0; index=0; succ=[] } ]
;;
let rec merge_node ?(cmp=minify) pleaf = function
| [] -> [ pleaf ]
| (hd :: tail) as s ->
if equal hd.data pleaf.data then
if hd.index < pleaf.index then
{ hd with succ=pleaf :: (hd.succ @ pleaf.succ) } :: s
else
{ pleaf with succ=hd :: (hd.succ @ pleaf.succ) } :: s
else if cmp hd.data pleaf.data then
{ hd with succ=(merge_node pleaf ~cmp:cmp hd.succ) } :: tail
else
hd :: (merge_node pleaf ~cmp:cmp tail)
;;
let dedup pleaf tree =
let rec fdup stree ntree acc = match stree with
| [] -> (ntree, acc)
| hd :: tl ->
if equal hd.data pleaf then
let (ntree', acc') = fdup hd.succ ntree (hd.data :: acc) in
fdup tl ntree' acc'
else
fdup tl (hd :: ntree) acc
in fdup tree [] []
;;
let dedup_idx pleaf tree =
let rec fdup stree ntree acc = match stree with
| [] -> (ntree, acc)
| hd :: tl ->
if equal hd.data pleaf then
let (ntree', acc') = fdup hd.succ ntree ((hd.data, hd.index) :: acc) in
fdup tl ntree' acc'
else
fdup tl (hd :: ntree) acc
in fdup tree [] []
;;
let merge ?(cmp=minify) tree trunk =
if equal tree.data trunk.data then
if tree.index > trunk.index then
{ trunk with succ = tree :: trunk.succ }
else
{ tree with succ = trunk :: tree.succ }
else if cmp tree.data trunk.data then
{ tree with succ=(merge_node ~cmp:cmp trunk tree.succ) }
else
{ trunk with succ=(merge_node ~cmp:cmp tree trunk.succ) }
;;
exception Empty
let degree tree = List.length tree.succ
;;
(** inorder traverse the heap, elements will likely be out of order
NB: In the rem, we use list of list rather than flat plain list to
speed up the process
*)
let collapse = function
| [] -> []
| { succ=child; data=pleaf;_ } :: tail ->
let rec fcollapse tree rem acc =
match tree with
| [] ->
(match rem with
| [] -> acc
| branch :: left ->
(match branch with
| [] ->
(match left with
| [] -> acc
| l :: r -> fcollapse l r acc)
| { succ=child''; data=pleaf'';_ } :: tl'' ->
fcollapse child'' ([ tl'' ] @ left) (pleaf'' :: acc)
)
)
| { succ=child'; data=pleaf';_ } :: tl' ->
fcollapse child' (tl' :: rem) (pleaf' :: acc)
in fcollapse child ([ tail ]) ([ pleaf ])
;;
let rec peek ?(cmp=minify) = function
| [] -> raise Empty
| hd :: tail ->
match tail with
| [] -> hd
| fllw :: rest ->
if cmp hd.data fllw.data then
peek ~cmp:cmp (hd :: rest)
else
peek ~cmp:cmp (fllw :: rest)
;;
let rec peek_opt ?(cmp=minify) = function
| [] -> None
| hd :: tail ->
match tail with
| [] -> Some hd
| fllw :: rest ->
if cmp hd.data fllw.data then
peek_opt ~cmp:cmp (hd :: rest)
else
peek_opt ~cmp:cmp (fllw :: rest)
;;
let consolidate ?(cmp=minify) trees =
let tbl = Hashtbl.create 8 in
let rec cascade rejoin =
let leftover = List.fold_left (fun acc eltree ->
let deg = degree eltree in
match Hashtbl.find_opt tbl deg with
| Some tree ->
let ntree = merge ~cmp:cmp tree eltree in
let _ = Hashtbl.remove tbl deg in
ntree :: acc
| None ->
let _ = Hashtbl.add tbl deg eltree in
acc
) [] rejoin in match leftover with
| [] -> ()
| _ :: _ -> cascade leftover
in
let _ = cascade trees in
let fin = List.of_seq @@ Hashtbl.to_seq_values tbl in
let _ = Hashtbl.clear tbl in
fin
;;
let ?(cmp=minify) = function
| [] -> raise Empty
| hd :: tail ->
let rec split hd tl acc =
match tl with
| [] -> (hd, acc)
| fllw :: rest ->
if cmp hd.data fllw.data then
split hd rest (fllw :: acc)
else
split fllw rest (hd :: acc)
in
let (it, rem) = split hd tail [] in
(it.data, consolidate ~cmp:cmp (rem @ it.succ))
;;
let ?(cmp=minify) = function
| [] -> None
| head :: tail ->
let rec split hd tl acc =
match tl with
| [] -> (hd, acc)
| fllw :: rest ->
if cmp hd.data fllw.data then
split hd rest (fllw :: acc)
else
split fllw rest (hd :: acc)
in
let (it, rem) = split head tail [] in
Some (it.data, consolidate ~cmp:cmp (rem @ it.succ))
;;
let rec ?(cmp=minify) tree =
let sml, rem = extract ~cmp:cmp tree in
match rem with
| [] -> [ sml ]
| rest -> sml :: extract_all ~cmp:cmp rest
;;
let of_list ?(cmp=minify) els =
List.fold_right (insert ~cmp:cmp) els empty
;;
let to_seq ?(cmp=minify) tree =
let rec aux l () = match extract_opt ~cmp:cmp l with
| None -> Seq.Nil
| Some (hd, tail) -> Seq.Cons (hd, (aux tail))
in
(aux tree)
;;
let rec of_seq ?(cmp=minify) tseq =
match tseq () with
| Seq.Nil -> []
| Seq.Cons (x1, seq) ->
begin match seq () with
| Seq.Nil -> [(instance x1 0)]
| Seq.Cons (x2, seq) ->
insert ~cmp:cmp (x1) @@ insert ~cmp:cmp (x2) @@ of_seq ~cmp:cmp seq
end
;;
let ?(cmp=minify) f tree =
let sml , rem = extract ~cmp:cmp tree in
if f sml then
[ sml ]
else
match rem with
| [] -> [ sml ]
| rest -> sml :: extract_all ~cmp:cmp rest
;;
let update ?(cmp=minify) newent hd tl parent leftover =
let maxbelopt = (peek_opt ~cmp:cmp hd.succ) in
match maxbelopt with
| Some maxbel ->
let par, son = (cmp parent.data newent), (cmp maxbel.data newent) in
if par && not son then
({ hd with data=newent } :: tl, leftover, true)
else
let _ = (parent.churn <- parent.churn + 1) in
(tl, { data=newent; churn=0; succ=[]; index=hd.index; } :: hd.succ, true)
| None ->
let par = (cmp parent.data newent) in
if par then
({ hd with data=newent } :: tl, leftover, true)
else
let _ = (parent.churn <- parent.churn + 1) in
(tl, { data=newent; churn=0; succ=[]; index=hd.index } :: hd.succ, true)
;;
let increase ?(cmp=minify) old bgger tree =
match tree with
| [] -> raise Empty
| _ :: _ ->
let rec atparent parent tree leftover found = match tree with
| [] ->
tree, leftover, found
| (hd :: tl) ->
if (equal hd.data old) then
if (oequal hd.data bgger) = 1 then
failwith "new value must be larger"
else
update ~cmp:cmp bgger hd tl parent leftover
else
let nt, lf, wasfound = atparent hd hd.succ leftover found in
if wasfound then
if hd.churn > (!churn_threshold) then
let _ = (parent.churn <- (parent.churn + 1)) in
(tl, { hd with succ=nt } :: lf, wasfound)
else
({ hd with succ=nt } :: tl, lf, wasfound)
else
let nt, lf, rem = atparent parent tl leftover found in
(hd :: nt, lf, rem)
in
let self = { data=bgger; churn=0; succ=[]; index=0; } in
let ntree, left, found = atparent self tree [] false in
if found then
let _ = (List.iter (fun n -> n.churn <- 0) left) in
ntree @ left
else
failwith "value not in heap"
;;
let decrease ?(cmp=minify) old smller tree =
match tree with
| [] -> raise Empty
| _ :: _ ->
let rec atparent parent tree leftover found = match tree with
| [] ->
(tree, leftover, found)
| (hd :: tl) ->
if (equal hd.data old) then
if (oequal hd.data smller) = -1 then
failwith "new value must be smaller"
else
update ~cmp:cmp smller hd tl parent leftover
else
let nt, lf, wasfound = (atparent hd hd.succ leftover found) in
if wasfound then
if hd.churn > (!churn_threshold) then
let _ = (parent.churn <- (parent.churn + 1)) in
(tl, { hd with succ=nt } :: lf, wasfound)
else
({ hd with succ=nt } :: tl, lf, wasfound)
else
let nt, lf, rem = atparent parent tl leftover found in
(hd :: nt, lf, rem)
in
let self = { data=smller; churn=0; succ=[]; index=0; } in
let ntree, left, wasfound = atparent self tree [] false in
if wasfound then
let _ = (List.iter (fun n -> n.churn <- 0) left) in
ntree @ left
else
failwith "value not in heap"
;;
end